require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.Context"
import "android.os.*"
import "cjson"

activity = this

local languages = {
    ["Indonesia"] = "id",
    ["Inggris"] = "en"
}

function vibrate()
    local vibrator = activity.getSystemService(Context.VIBRATOR_SERVICE)
    if vibrator then
        vibrator.vibrate(50)
    end
end

function showToast(message)
    Toast.makeText(activity, message, Toast.LENGTH_SHORT).show()
end

-- Save configuration
local function saveConfig(config)
    local prefs = activity.getSharedPreferences("appConfig", Context.MODE_PRIVATE)
    local editor = prefs.edit()
    editor.putBoolean("vibrate", config.vibrate)
    editor.putBoolean("correction", config.correction)
    editor.putBoolean("translate", config.translate)
    editor.putBoolean("sound", config.sound)
    editor.putBoolean("autoSend", config.autoSend)
    editor.putBoolean("tts", config.tts) -- Save TTS configuration
    editor.putString("sourceLang", config.sourceLang)
    editor.putString("translateLang", config.translateLang)
    editor.putString("correctionMode", config.correctionMode)
    editor.putString("aiModel", config.aiModel)
    editor.putString("textEnding", config.textEnding)
    editor.putString("customInstruction", config.customInstruction)
    editor.putString("customSendText", config.customSendText)
    editor.apply()
end

-- Load configuration
local function loadConfig()
    local prefs = activity.getSharedPreferences("appConfig", Context.MODE_PRIVATE)
    return {
        vibrate = prefs.getBoolean("vibrate", true),
        correction = prefs.getBoolean("correction", true),
        translate = prefs.getBoolean("translate", false),
        sound = prefs.getBoolean("sound", true),
        autoSend = prefs.getBoolean("autoSend", false),
        tts = prefs.getBoolean("tts", false), -- Load TTS configuration
        sourceLang = prefs.getString("sourceLang", "id"),
        translateLang = prefs.getString("translateLang", "en"),
        correctionMode = prefs.getString("correctionMode", "punctuation"),
        aiModel = prefs.getString("aiModel", "openai"),
        textEnding = prefs.getString("textEnding", "nothing"),
        customInstruction = prefs.getString("customInstruction", ""),
        customSendText = prefs.getString("customSendText", "Kirim") -- Default "Kirim" jika kosong
    }
end

local config = loadConfig() -- Inisialisasi konfigurasi

-- Fungsi untuk menampilkan dialog input custom
local function showCustomDialog(title, hint, key, config)
    local dialog
    local layout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "16dp",
        {
            EditText,
            id = "inputField",
            hint = hint,
            layout_width = "fill",
            layout_height = "wrap_content",
            text = config[key] -- Isi dengan nilai lama
        },
        {
            Button,
            text = "Simpan",
            layout_width = "fill",
            onClick = function()
                local input = inputField.getText().toString()
                if input ~= "" then -- Validasi input tidak kosong
                    config[key] = input
                    saveConfig(config) -- Simpan perubahan ke SharedPreferences
                    showToast(title .. " berhasil disimpan!")
                    dialog.dismiss()
                else
                    showToast("Teks tidak boleh kosong!")
                end
            end
        },
        {
            Button,
            text = "Batal",
            layout_width = "fill",
            onClick = function()
                dialog.dismiss()
            end
        }
    }
dialog = LuaDialog(activity)
    dialog.setTitle(title)
    dialog.setView(loadlayout(layout))
    dialog.show()
end

local function getLanguageNameByCode(code)
    for name, langCode in pairs(languages) do
        if langCode == code then
            return name
        end
    end
    return nil
end

local function getLanguageCodeByName(name)
    return languages[name]
end

-- Dialog Layout
local layout = {
    ScrollView,
    layout_width = "fill",
    layout_height = "wrap_content",
    {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        padding = "16dp",
        layout_height = "wrap_content",
        {
            CheckBox,
            id = "vibrateCheckbox",
            text = "Vibrate",
        },
        {
            CheckBox,
            id = "soundCheckbox",
            text = "Sound effect",
        },
        {
            CheckBox,
            id = "autoSendCheckbox",
            text = "Auto Send",
        },
        {
            CheckBox,
            id = "ttsCheckbox", -- Checkbox untuk TTS
            text = "Text-to-Speech",
        },
        {
            CheckBox,
            id = "correctionCheckbox",
            text = "Correction",
        },
        {
            Spinner,
            id = "modelSpinner",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            Spinner,
            id = "correctionModeSpinner",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            Button,
            text = "Custom Instruksi",
            layout_width = "fill",
            layout_height = "wrap_content",
            onClick = function()
                showCustomDialog("Custom Instruksi", "Masukkan instruksi kustom Anda", "customInstruction", config)
            end
        },
        {
            Button,
            text = "Custom Tombol Kirim",
            layout_width = "fill",
            layout_height = "wrap_content",
            onClick = function()
                showCustomDialog("Custom Tombol Kirim", "Masukkan teks tombol kirim kustom Anda", "customSendText", config)
            end
        },
        {
            CheckBox,
            id = "translateCheckbox",
            text = "Translate",
        },
        {
            Spinner,
            id = "sourceLangSpinner",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            Spinner,
            id = "translateLangSpinner",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            Spinner,
            id = "textEndingSpinner",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            Button,
            id = "saveButton",
            text = "Save Configuration",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            Button,
            id = "closeButton",
            text = "Close",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
    }
}

dlg = LuaDialog(activity)
dlg.setTitle("App Configuration")
dlg.setView(loadlayout(layout))

-- Initialize configuration
vibrateCheckbox.setChecked(config.vibrate)
soundCheckbox.setChecked(config.sound)
autoSendCheckbox.setChecked(config.autoSend)
ttsCheckbox.setChecked(config.tts) -- Inisialisasi checkbox TTS
correctionCheckbox.setChecked(config.correction)
translateCheckbox.setChecked(config.translate)

local modelOptions = {"openai", "mistralNemo", "saanviAI", "gemini", "doppleAI", "LilyAI", "deepseek-coder-67b", "deepseek-r1"}
local modelAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, modelOptions)
modelAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
modelSpinner.setAdapter(modelAdapter)
modelSpinner.setSelection(modelAdapter.getPosition(config.aiModel))

local correctionModes = {"punctuation", "emoji", "shortText", "custom"}
local correctionModeAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, correctionModes)
correctionModeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
correctionModeSpinner.setAdapter(correctionModeAdapter)
correctionModeSpinner.setSelection(correctionModeAdapter.getPosition(config.correctionMode))

local languageOptions = {"Indonesia", "Inggris"}
local languageAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, languageOptions)
languageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
sourceLangSpinner.setAdapter(languageAdapter)
translateLangSpinner.setAdapter(languageAdapter)
sourceLangSpinner.setSelection(languageAdapter.getPosition(getLanguageNameByCode(config.sourceLang)))
translateLangSpinner.setSelection(languageAdapter.getPosition(getLanguageNameByCode(config.translateLang)))

local textEndings = {"new line", "space", "dot", "nothing"}
local textEndingAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, textEndings)
textEndingAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
textEndingSpinner.setAdapter(textEndingAdapter)
textEndingSpinner.setSelection(textEndingAdapter.getPosition(config.textEnding))

-- Save button handler
saveButton.setOnClickListener(function()
    config.vibrate = vibrateCheckbox.isChecked()
    config.sound = soundCheckbox.isChecked()
    config.autoSend = autoSendCheckbox.isChecked()
    config.tts = ttsCheckbox.isChecked() -- Simpan status TTS
    config.correction = correctionCheckbox.isChecked()
    config.translate = translateCheckbox.isChecked()
    config.autoSend = autoSendCheckbox.isChecked()
    config.sourceLang = getLanguageCodeByName(sourceLangSpinner.getSelectedItem())
    config.correctionMode = correctionModeSpinner.getSelectedItem()
    config.aiModel = modelSpinner.getSelectedItem()
    config.textEnding = textEndingSpinner.getSelectedItem()
    saveConfig(config) -- Simpan ke SharedPreferences
    showToast("Configuration saved!")
    dlg.dismiss()
end)

closeButton.setOnClickListener(function()
vibrate()
    dlg.dismiss()
end)

service.playSoundTick()
vibrate()
dlg.show()


